import 'package:flutter/widgets.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:info_tren/api/train_data.dart';
import 'package:info_tren/components/loading/loading.dart';
import 'package:info_tren/components/refresh_future_builder.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_cupertino.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_fluent.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_material.dart';
import 'package:info_tren/providers.dart';


class TrainInfo extends ConsumerWidget {
  static String routeName = "/trainInfo/display";

  const TrainInfo({super.key,});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final uiDesign = ref.watch(uiDesignProvider);
    final args = ref.watch(trainInfoArgumentsProvider);
    final trainNumber = args.trainNumber;
    final date = args.date;

    return RefreshFutureBuilder<TrainData>(
      futureCreator: () => getTrain(trainNumber, date: date),
      builder: (context, refresh, replaceFutureBuilder, snapshot) {
        void onViewYesterdayTrain() {
          replaceFutureBuilder(() => getTrain(trainNumber, date: DateTime.now().subtract(const Duration(days: 1))));
        }

        if ([RefreshFutureBuilderState.none, RefreshFutureBuilderState.waiting].contains(snapshot.state)) {
          return TrainInfoLoading(title: trainNumber.toString(), loadingText: "Se încarcă...",);
        }
        else if (snapshot.state == RefreshFutureBuilderState.error) {
          return TrainInfoError(title: '$trainNumber - Error', error: snapshot.error!, refresh: refresh,);
        }

        switch (uiDesign) {
          case UiDesign.MATERIAL:
            return TrainInfoMaterial(
              trainData: snapshot.data!, 
              refresh: refresh,
              isRefreshing: snapshot.state == RefreshFutureBuilderState.refreshing,
              onViewYesterdayTrain: onViewYesterdayTrain,
            );
          case UiDesign.CUPERTINO:
            return TrainInfoCupertino(
              trainData: snapshot.data!, 
              refresh: refresh, 
              isRefreshing: snapshot.state == RefreshFutureBuilderState.refreshing,
              onViewYesterdayTrain: onViewYesterdayTrain,
            );
          case UiDesign.FLUENT:
            return TrainInfoFluent(
              trainData: snapshot.data!,
              refresh: refresh,
              isRefreshing: snapshot.state == RefreshFutureBuilderState.refreshing,
              onViewYesterdayTrain: onViewYesterdayTrain,
            );
          default:
            throw UnmatchedUiDesignException(uiDesign);
        }
      },
    );
  }
}

class TrainInfoArguments {
  final String trainNumber;
  final DateTime? date;

  TrainInfoArguments({required this.trainNumber, this.date});
}

abstract class TrainInfoShared extends StatelessWidget {
  final TrainData trainData;
  final Future Function()? refresh;
  final void Function()? onViewYesterdayTrain;
  final bool? isRefreshing;

  const TrainInfoShared({
    super.key,
    required this.trainData,
    this.refresh,
    this.onViewYesterdayTrain,
    this.isRefreshing,
  });
}

class TrainInfoLoading extends ConsumerWidget {
  final String title;
  final String? loadingText;

  const TrainInfoLoading({
    super.key,
    required this.title,
    this.loadingText,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final uiDesign = ref.watch(uiDesignProvider);

    switch (uiDesign) {
      case UiDesign.MATERIAL:
        return TrainInfoLoadingMaterial(
          title: title,
          loadingText: loadingText,
        );
      case UiDesign.CUPERTINO:
        return TrainInfoLoadingCupertino(
          title: title,
          loadingText: loadingText,
        );
      case UiDesign.FLUENT:
        return TrainInfoLoadingFluent(
          title: title,
          loadingText: loadingText,
        );
      default:
        throw UnmatchedUiDesignException(uiDesign);
    }
  }
}

abstract class TrainInfoLoadingShared extends StatelessWidget {
  final String title;
  final Widget loadingWidget;

  TrainInfoLoadingShared({
    required this.title, 
    String? loadingText, 
    super.key,
  }) : loadingWidget = Loading(text: loadingText,);
}

class TrainInfoError extends ConsumerWidget {
  final String title;
  final Object error;
  final Future Function()? refresh;

  const TrainInfoError({
    super.key,
    required this.title,
    required this.error,
    this.refresh,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final uiDesign = ref.watch(uiDesignProvider);

    switch (uiDesign) {
      case UiDesign.MATERIAL:
        return TrainInfoErrorMaterial(
          title: title,
          error: error,
          refresh: refresh,
        );
      case UiDesign.CUPERTINO:
        return TrainInfoErrorCupertino(
          title: title,
          error: error,
          refresh: refresh,
        );
      case UiDesign.FLUENT:
        return TrainInfoErrorFluent(
          title: title,
          error: error,
          refresh: refresh,
        );
      default:
        throw UnmatchedUiDesignException(uiDesign);
    }
  }
}

abstract class TrainInfoErrorShared extends StatelessWidget {
  final String title;
  final Object error;
  final Future Function()? refresh;

  const TrainInfoErrorShared({required this.title, required this.error, this.refresh, super.key,});
}

class TrainInfoBody extends ConsumerWidget {
  final TrainData trainData;
  final void Function()? onViewYesterdayTrain;
  final Future Function()? refresh;
  final bool? isRefreshing;

  const TrainInfoBody({
    required this.trainData,
    this.onViewYesterdayTrain,
    this.refresh,
    this.isRefreshing,
    super.key,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final uiDesign = ref.watch(uiDesignProvider);

    switch (uiDesign) {
      case UiDesign.MATERIAL:
        return TrainInfoBodyMaterial(
          trainData: trainData,
          onViewYesterdayTrain: onViewYesterdayTrain,
          refresh: refresh,
          isRefreshing: isRefreshing,
        );
      case UiDesign.CUPERTINO:
        return TrainInfoBodyCupertino(
          trainData: trainData,
          onViewYesterdayTrain: onViewYesterdayTrain,
          refresh: refresh,
          isRefreshing: isRefreshing,
        );
      case UiDesign.FLUENT:
        return TrainInfoBodyFluent(
          trainData: trainData,
          onViewYesterdayTrain: onViewYesterdayTrain,
          refresh: refresh,
          isRefreshing: isRefreshing,
        );
      default:
        throw UnmatchedUiDesignException(uiDesign);
    }
  }
}

abstract class TrainInfoBodyShared extends StatelessWidget {
  final TrainData trainData;
  final void Function()? onViewYesterdayTrain;
  final Future Function()? refresh;
  final bool? isRefreshing;

  const TrainInfoBodyShared({
    required this.trainData,
    this.onViewYesterdayTrain,
    this.refresh,
    this.isRefreshing,
    super.key,
  });
}

abstract class DisplayTrainYesterdayWarningCommon extends StatelessWidget {
  static const trainDidNotDepart = 'Acest tren nu a plecat încă din prima gară.';
  static const seeYesterdayTrain = 'Apasă aici pentru a vedea trenul care a plecat ieri.';

  final void Function() onViewYesterdayTrain;

  const DisplayTrainYesterdayWarningCommon(this.onViewYesterdayTrain, {super.key,});
}
